declare type ArgumentsType<T> = T extends (...args: infer A) => any ? A : never;
declare type ReturnType<T> = T extends (...args: any) => infer R ? R : never;
interface BirpcOptions<S> {
    /**
     * Local functions implementation.
     */
    functions: S;
    /**
     * Function to post raw message
     */
    post: (data: any) => void;
    /**
     * Listener to receive raw message
     */
    on: (fn: (data: any) => void) => void;
    /**
     * Custom function to serialize data
     *
     * by default it passes the data as-is
     */
    serialize?: (data: any) => any;
    /**
     * Custom function to deserialize data
     *
     * by default it passes the data as-is
     */
    deserialize?: (data: any) => any;
}
interface BirpcReturn<RemoteFunctions> {
    /**
     * Call a remote function and wait for the result.
     */
    call<T extends keyof RemoteFunctions>(method: T, ...args: ArgumentsType<RemoteFunctions[T]>): Promise<Awaited<ReturnType<RemoteFunctions[T]>>>;
    /**
     * Send events without waiting for response
     */
    send<T extends keyof RemoteFunctions>(method: T, ...args: ArgumentsType<RemoteFunctions[T]>): void;
}
declare function createBirpc<LocalFunctions = {}, RemoteFunctions = {}>({ functions, post, on, serialize, deserialize, }: BirpcOptions<LocalFunctions>): BirpcReturn<RemoteFunctions>;

export { ArgumentsType, BirpcOptions, BirpcReturn, ReturnType, createBirpc };
