function createBirpc({
  functions,
  post,
  on,
  serialize = (i) => i,
  deserialize = (i) => i
}) {
  const rpcPromiseMap = /* @__PURE__ */ new Map();
  on(async (data) => {
    const msg = deserialize(data);
    if (msg.type === "req") {
      const { method, args, ack } = msg;
      let result, error;
      try {
        result = await functions[method](...args);
      } catch (e) {
        error = e;
      }
      if (ack)
        await post(serialize({ type: "res", ack, result, error }));
    } else {
      const { ack, result, error } = msg;
      const promise = rpcPromiseMap.get(ack);
      if (error)
        promise?.reject(error);
      else
        promise?.resolve(result);
      rpcPromiseMap.delete(ack);
    }
  });
  return {
    call(method, ...args) {
      return new Promise((resolve, reject) => {
        const ack = nanoid();
        rpcPromiseMap.set(ack, { resolve, reject });
        post(serialize({ method, args, ack, type: "req" }));
      });
    },
    send(method, ...args) {
      post(serialize({ method, args, type: "req" }));
    }
  };
}
const urlAlphabet = "useandom-26T198340PX75pxJACKVERYMINDBUSHWOLF_GQZbfghjklqvwyzrict";
function nanoid(size = 21) {
  let id = "";
  let i = size;
  while (i--)
    id += urlAlphabet[Math.random() * 64 | 0];
  return id;
}

export { createBirpc };
